@@ -1,481 +1,18 @@
-## no critic
-package Make::Rule::Vars;
-use strict;
-use warnings;
-
-use Carp;
-
-my $generation = 0;    # lexical cross-package scope used!
-
-# Package to handle 'magic' variables pertaining to rules e.g. $@ $* $^ $?
-# by using tie to this package 'subsvars' can work with array of
-# hash references to possible sources of variable definitions.
-
-sub TIEHASH {
-	my ( $class, $rule ) = @_;
-	return bless \$rule, $class;
-}
-
-sub FETCH {
-	my $self = shift;
-	local $_ = shift;
-	my $rule = $$self;
-	return unless (/^[\@^<?*]$/);
-
-	# print STDERR "FETCH $_ for ",$rule->Name,"\n";
-	return $rule->Name if ( $_ eq '@' );
-	return $rule->Base if ( $_ eq '*' );
-	return join( ' ', $rule->exp_depend )  if ( $_ eq '^' );
-	return join( ' ', $rule->out_of_date ) if ( $_ eq '?' );
-
-	# Next one is dubious - I think $< is really more subtle ...
-	return ( $rule->exp_depend )[0] if ( $_ eq '<' );
-	return;
-}
-
-## no critic
-package Make::Rule;
-use strict;
-use warnings;
-
-use Carp;
-
-# Bottom level 'rule' package
-# An instance exists for each ':' or '::' rule in the makefile.
-# The commands and dependancies are kept here.
-
-sub target {
-	return shift->{TARGET};
-}
-
-sub Name {
-	return shift->target->Name;
-}
-
-sub Base {
-	my $name = shift->target->Name;
-	$name =~ s/\.[^.]+$//;
-	return $name;
-}
-
-sub Info {
-	return shift->target->Info;
-}
-
-sub depend {
-	my $self = shift;
-	if (@_) {
-		my $name = $self->Name;
-		my $dep  = shift;
-		confess "dependants $dep are not an array reference" unless ( 'ARRAY' eq ref $dep );
-		foreach my $file (@$dep) {
-			unless ( exists $self->{DEPHASH}{$file} ) {
-				$self->{DEPHASH}{$file} = 1;
-				push( @{ $self->{DEPEND} }, $file );
-			}
-		}
-	}
-	return (wantarray) ? @{ $self->{DEPEND} } : $self->{DEPEND};
-}
-
-sub command {
-	my $self = shift;
-	if (@_) {
-		my $cmd = shift;
-		confess "commands $cmd are not an array reference" unless ( 'ARRAY' eq ref $cmd );
-		if (@$cmd) {
-			if ( @{ $self->{COMMAND} } ) {
-				warn "Command for " . $self->Name, " redefined";
-				print STDERR "Was:", join( "\n", @{ $self->{COMMAND} } ), "\n";
-				print STDERR "Now:", join( "\n", @$cmd ), "\n";
-			}
-			$self->{COMMAND} = $cmd;
-		}
-		else {
-			if ( @{ $self->{COMMAND} } ) {
-
-				# warn "Command for ".$self->Name," retained";
-				# print STDERR "Was:",join("\n",@{$self->{COMMAND}}),"\n";
-			}
-		}
-	}
-	return (wantarray) ? @{ $self->{COMMAND} } : $self->{COMMAND};
-}
-
-#
-# The key make test - is target out-of-date as far as this rule is concerned
-# In scalar context - boolean value of 'do we need to apply the rule'
-# In list context the things we are out-of-date with e.g. magic $? variable
-#
-sub out_of_date {
-	my $array = wantarray;
-	my $self  = shift;
-	my $info  = $self->Info;
-	my @dep   = ();
-	my $tdate = $self->target->date;
-	my $count = 0;
-	foreach my $dep ( $self->exp_depend ) {
-
-		# This is a dumb fix around regex issues with using Strawberry perl.
-		# This may not fix for all versions of Strawberry perl or all versions
-		# of windows, but is a hacky work-around until a real fix can be implemented
-		if ( $^O eq 'MSWin32' ) {
-			if ( $dep =~ m/libConfig.pm/ ) {
-				$dep =~ s#libConfig.pm#lib\\Config.pm#;
-			}
-			if ( $dep =~ m/COREconfig.h/ ) {
-				$dep =~ s#COREconfig.h#CORE\\config.h#;
-			}
-		}
-		my $date = $info->date($dep);
-		$count++;
-		if ( !defined($date) || !defined($tdate) || $date < $tdate ) {
-
-			# warn $self->Name." ood wrt ".$dep."\n";
-			return 1 unless $array;
-			push( @dep, $dep );
-		}
-	}
-	return @dep if $array;
-
-	# Note special case of no dependencies means it is always  out-of-date!
-	return !$count;
-}
-
-#
-# Return list of things rule depends on with variables expanded
-# - May need pathname and vpath processing as well
-#
-sub exp_depend {
-	my $self = shift;
-	my $info = $self->Info;
-	my @dep  = map( split( /\s+/, $info->subsvars($_) ), $self->depend );
-	return (wantarray) ? @dep : \@dep;
-}
-
-#
-# Return commands to apply rule with variables expanded
-# - No pathname processing needed, commands should always chdir()
-#   to logical place (at least till we get very clever at bourne shell parsing).
-# - May need vpath processing
-#
-sub exp_command {
-	my $self = shift;
-	my $info = $self->Info;
-	my $base = $self->Name;
-	my %var;
-	tie %var, 'Make::Rule::Vars', $self;
-	my @cmd = map( $info->subsvars( $_, \%var ), $self->command );
-	return (wantarray) ? @cmd : \@cmd;
-}
-
-#
-# clone creates a new rule derived from an existing rule, but
-# with a different target. Used when left hand side was a variable.
-# perhaps should be used for dot/pattern rule processing too.
-#
-sub clone {
-	my ( $self, $target ) = @_;
-	my %hash = %$self;
-	$hash{TARGET}  = $target;
-	$hash{DEPEND}  = [ @{ $self->{DEPEND} } ];
-	$hash{DEPHASH} = { %{ $self->{DEPHASH} } };
-	my $obj = bless \%hash, ref $self;
-	return $obj;
-}
-
-sub new {
-	my $class  = shift;
-	my $target = shift;
-	my $kind   = shift;
-	my $self   = bless {
-		TARGET  => $target,              # parent target (left hand side)
-		KIND    => $kind,                # : or ::
-		DEPEND  => [], DEPHASH => {},    # right hand args
-		COMMAND => []                    # command(s)
-	}, $class;
-	$self->depend(shift)  if (@_);
-	$self->command(shift) if (@_);
-	return $self;
-}
-
-#
-# This code has to go somewhere but no good home obvious yet.
-#  - only applies to ':' rules, but needs top level database
-#  - perhaps in ->commands of derived ':' class?
-#
-sub find_commands {
-	my ($self) = @_;
-	if ( !@{ $self->{COMMAND} } && @{ $self->{DEPEND} } ) {
-		my $info = $self->Info;
-		my $name = $self->Name;
-		my @dep  = $self->depend;
-		my @rule = $info->patrule( $self->Name );
-		if (@rule) {
-			$self->depend( $rule[0] );
-			$self->command( $rule[1] );
-		}
-	}
-}
-
-#
-# Spew a shell script to perfom the 'make' e.g. make -n
-#
-sub Script {
-	my $self = shift;
-	return unless $self->out_of_date;
-	my @cmd = $self->exp_command;
-	if (@cmd) {
-		my $com = ( $^O eq 'MSWin32' ) ? 'rem ' : '# ';
-		print $com, $self->Name, "\n";
-		foreach my $file ( $self->exp_command ) {
-			$file =~ s/^[\@\s-]*//;
-			print "$file\n";
-		}
-	}
-}
-
-#
-# Normal 'make' method
-#
-sub Make {
-	my $self = shift;
-	my $file;
-	return unless ( $self->out_of_date );
-	my @cmd  = $self->exp_command;
-	my $info = $self->Info;
-	if (@cmd) {
-		foreach my $file ( $self->exp_command ) {
-			$file =~ s/^([\@\s-]*)//;
-			my $prefix = $1;
-			print "$file\n" unless ( $prefix =~ /\@/ );
-			my $code = $info->exec($file);
-			if ( $code && $prefix !~ /-/ ) {
-				die "Code $code from $file";
-			}
-		}
-	}
-}
-
-#
-# Print rule out in makefile syntax
-# - currently has variables expanded as debugging aid.
-# - will eventually become make -p
-# - may be useful for writing makefiles from MakeMaker too...
-#
-sub Print {
-	my $self = shift;
-	my $file;
-	print $self->Name, ' ', $self->{KIND}, ' ';
-	foreach my $file ( $self->depend ) {
-		print " \\\n   $file";
-	}
-	print "\n";
-	my @cmd = $self->exp_command;
-	if (@cmd) {
-		foreach my $file ( $self->exp_command ) {
-			print "\t", $file, "\n";
-		}
-	}
-	else {
-		print STDERR "No commands for ", $self->Name, "\n" unless ( $self->target->phony );
-	}
-	print "\n";
-}
-
-package Make::Target;
-use strict;
-use warnings;
-
-use Carp;
-use Cwd;
-
-#
-# Intermediate 'target' package
-# There is an instance of this for each 'target' that apears on
-# the left hand side of a rule i.e. for each thing that can be made.
-#
-sub new {
-	my ( $class, $info, $target ) = @_;
-	return bless {
-		NAME     => $target,    # name of thing
-		MAKEFILE => $info,      # Makefile context
-		Pass     => 0           # Used to determine if 'done' this sweep
-	}, $class;
-}
-
-sub date {
-	my $self = shift;
-	my $info = $self->Info;
-	return $info->date( $self->Name );
-}
-
-sub phony {
-	my $self = shift;
-	return $self->Info->phony( $self->Name );
-}
-
-sub colon {
-	my $self = shift;
-	if (@_) {
-		if ( exists $self->{COLON} ) {
-			my $dep = $self->{COLON};
-			if ( @_ == 1 ) {
-
-				# merging an existing rule
-				my $other = shift;
-				$dep->depend( scalar $other->depend );
-				$dep->command( scalar $other->command );
-			}
-			else {
-				$dep->depend(shift);
-				$dep->command(shift);
-			}
-		}
-		else {
-			$self->{COLON} = ( @_ == 1 ) ? shift->clone($self) : Make::Rule->new( $self, ':', @_ );
-		}
-	}
-	if ( exists $self->{COLON} ) {
-		return (wantarray) ? ( $self->{COLON} ) : $self->{COLON};
-	}
-	else {
-		return (wantarray) ? () : undef;
-	}
-}
-
-sub dcolon {
-	my $self = shift;
-	if (@_) {
-		my $rule = ( @_ == 1 ) ? shift->clone($self) : Make::Rule->new( $self, '::', @_ );
-		$self->{DCOLON} = [] unless ( exists $self->{DCOLON} );
-		push( @{ $self->{DCOLON} }, $rule );
-	}
-	return ( exists $self->{DCOLON} ) ? @{ $self->{DCOLON} } : ();
-}
-
-sub Name {
-	return shift->{NAME};
-}
-
-sub Info {
-	return shift->{MAKEFILE};
-}
-
-sub ProcessColon {
-	my ($self) = @_;
-	my $c = $self->colon;
-	$c->find_commands if $c;
-}
-
-sub ExpandTarget {
-	my ($self) = @_;
-	my $target = $self->Name;
-	my $info   = $self->Info;
-	my $colon  = delete $self->{COLON};
-	my $dcolon = delete $self->{DCOLON};
-	foreach my $expand ( split( /\s+/, $info->subsvars($target) ) ) {
-		next unless defined($expand);
-		my $t = $info->Target($expand);
-		if ( defined $colon ) {
-			$t->colon($colon);
-		}
-		foreach my $d ( @{$dcolon} ) {
-			$t->dcolon($d);
-		}
-	}
-}
-
-sub done {
-	my $self = shift;
-	my $info = $self->Info;
-	my $pass = $info->pass;
-	return 1 if ( $self->{Pass} == $pass );
-	$self->{Pass} = $pass;
-	return 0;
-}
-
-sub recurse {
-	my ( $self, $method, @args ) = @_;
-	my $info = $self->Info;
-	my $i    = 0;
-	foreach my $rule ( $self->colon, $self->dcolon ) {
-		my $j = 0;
-		foreach my $dep ( $rule->exp_depend ) {
-
-			# This is a dumb fix around regex issues with using Strawberry perl.
-			# This may not fix for all versions of Strawberry perl or all versions
-			# of windows, but is a hacky work-around until a real fix can be implemented
-			if ( ( $dep =~ m/libConfig.pm/ ) and ( $^O eq 'MSWin32' ) ) {
-				print STDERR "we got here with $dep\n";
-				$dep =~ s#libConfig.pm#lib//Config.pm#;
-			}
-			if ( ( $dep =~ m/COREconfig.h/ ) and ( $^O eq 'MSWin32' ) ) {
-				print STDERR "we got here with $dep\n";
-				$dep =~ s#COREconfig.h#CORE//config.h#;
-			}
-
-			my $t = $info->{Depend}{$dep};
-			if ( defined $t ) {
-				$t->$method(@args);
-			}
-			else {
-				unless ( $info->exists($dep) ) {
-					my $dir = cwd();
-					die "Cannot recurse $method - no target $dep in $dir";
-				}
-			}
-		}
-	}
-}
-
-sub Script {
-	my $self = shift;
-	my $info = $self->Info;
-	my $rule = $self->colon;
-	return if ( $self->done );
-	$self->recurse('Script');
-	foreach my $rule ( $self->colon, $self->dcolon ) {
-		$rule->Script;
-	}
-}
-
-sub Make {
-	my $self = shift;
-	my $info = $self->Info;
-	my $rule = $self->colon;
-	return if ( $self->done );
-	$self->recurse('Make');
-	foreach my $rule ( $self->colon, $self->dcolon ) {
-		$rule->Make;
-	}
-}
-
-sub Print {
-	my $self = shift;
-	my $info = $self->Info;
-	return if ( $self->done );
-	my $rule = $self->colon;
-	foreach my $rule ( $self->colon, $self->dcolon ) {
-		$rule->Print;
-	}
-	$self->recurse('Print');
-}
-
 package Make;
 
-use 5.005;    # Need look-behind assertions
 use strict;
 use warnings;
 
+our $VERSION = '1.1.4';
+
 use Carp;
 use Config;
 use Cwd;
 use File::Spec;
-use vars qw($VERSION);
-$VERSION = '1.1.2';
+use Make::Target ();
 
 my %date;
+my $generation = 0;    # lexical cross-package scope used!
 
 sub phony {
 	my ( $self, $name ) = @_;
@@ -579,6 +116,7 @@ sub dotrules {
 	foreach my $t ( keys %{ $self->{Dot} } ) {
 		push( @{ $self->{Targets} }, delete $self->{Dot}{$t} );
 	}
+	return;
 }
 
 #
@@ -621,7 +159,9 @@ sub date {
 # file - used to see if pattern rules are valid
 # - Needs extending to do vpath lookups
 #
+## no critic (Subroutines::ProhibitBuiltinHomonyms)
 sub exists {
+## use critic
 	my ( $self, $name ) = @_;
 	return 1 if ( exists $self->{Depend}{$name} );
 	return 1 if defined $self->date($name);
@@ -684,6 +224,7 @@ sub needs {
 			}
 		}
 	}
+	return;
 }
 
 #
@@ -692,14 +233,18 @@ sub needs {
 # - recurses till they all go rather than doing one level,
 #   which may need fixing
 #
+## no critic (RequireArgUnpacking)
 sub subsvars {
 	my $self = shift;
 	local $_ = shift;
 	my @var = @_;
+## use critic
 	push( @var, $self->{Override}, $self->{Vars}, \%ENV );
 	croak("Trying to subsitute undef value") unless ( defined $_ );
+	## no critic (Variables::ProhibitMatchVars)
 	while ( /(?<!\$)\$\(([^()]+)\)/ || /(?<!\$)\$([<\@^?*])/ ) {
 		my ( $key, $head, $tail ) = ( $1, $`, $' );
+		## use critic
 		my $value;
 		if ( $key =~ /^([\w._]+|\S)(?::(.*))?$/ ) {
 			my ( $var, $op ) = ( $1, $2 );
@@ -737,7 +282,7 @@ sub subsvars {
 			$value = join( ' ', split( '\n', `$1` ) );
 		}
 		elsif ( $key =~ /addprefix\s*([^,]*),(.*)$/ ) {
-			$value = join( ' ', map( $1 . $_, split( '\s+', $2 ) ) );
+			$value = join( ' ', map { $1 . $_ } split( '\s+', $2 ) );
 		}
 		elsif ( $key =~ /notdir\s*(.*)$/ ) {
 			my @files = split( /\s+/, $1 );
@@ -801,6 +346,7 @@ sub tokenize {
 		last unless (/^\S/);
 		my $token = "";
 		while (/^\S/) {
+			## no critic (Variables::ProhibitMatchVars)
 			if (s/^\$([\(\{])//) {
 				$token .= $&;
 				my $paren = $1 eq '(';
@@ -820,6 +366,7 @@ sub tokenize {
 			elsif (s/^(\$\S?|[^\s\$]+)//) {
 				$token .= $&;
 			}
+			## use critic
 		}
 		push( @result, $token );
 	}
@@ -912,6 +459,7 @@ Makefile:
 			warn "Ignore '$_'\n";
 		}
 	}
+	return;
 }
 
 sub pseudos {
@@ -925,6 +473,7 @@ sub pseudos {
 			}
 		}
 	}
+	return;
 }
 
 sub ExpandTarget {
@@ -935,6 +484,7 @@ sub ExpandTarget {
 	foreach my $t ( @{ $self->{'Targets'} } ) {
 		$t->ProcessColon;
 	}
+	return;
 }
 
 sub parse {
@@ -961,6 +511,7 @@ sub parse {
 
 	$self->pseudos;     # Pull out .SUFFIXES etc.
 	$self->dotrules;    # Convert .c.o into %.o : %.c
+	return;
 }
 
 sub PrintVars {
@@ -970,6 +521,7 @@ sub PrintVars {
 		print "$_ = ", $self->{Vars}{$_}, "\n";
 	}
 	print "\n";
+	return;
 }
 
 sub exec {
@@ -1001,9 +553,12 @@ sub exec {
 	}
 }
 
+## no critic (Subroutines::RequireFinalReturn)
 sub NextPass { shift->{Pass}++ }
 sub pass     { shift->{Pass} }
+## use critic
 
+## no critic (RequireArgUnpacking)
 sub apply {
 	my $self   = shift;
 	my $method = shift;
@@ -1037,8 +592,11 @@ sub apply {
 		}
 		$t->$method();
 	}
+	return;
 }
+## use critic
 
+## no critic (Subroutines::RequireFinalReturn RequireArgUnpacking)
 sub Script {
 	shift->apply( Script => @_ );
 }
@@ -1050,6 +608,7 @@ sub Print {
 sub Make {
 	shift->apply( Make => @_ );
 }
+## use critic
 
 sub new {
 	my ( $class, %args ) = @_;
@@ -1160,6 +719,13 @@ L<pmake>
 
 Nick Ing-Simmons
 
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (c) 1996-1999 Nick Ing-Simmons.
+
+This program is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
 =cut
 
 1;
